from microprediction.reader import MicroReader
import muid, requests

class MicroWriter(MicroReader):

    def __init__(self, write_key="invalid_key", base_url="http://www.microprediction.com/"):
        """ Create the ability to write """
        super().__init__(base_url=base_url)
        assert muid.validate(write_key), "Invalid write_key. Mine one at muid.org. "
        self.write_key = write_key

    def __repr__(self):
        return {'write_key':self.write_key,"animal":muid.animal(self.write_key)}

    def set(self, name, value):
        """ Create or update a stream """
        res = requests.put(self.base_url + '/live/' + name, data={"write_key": self.write_key, "value": value})
        if res.status_code == 200:
            return float(res)

    def get_errors(self):
        """ Retrieve private log information """
        res = requests.get(self.base_url + 'live/errors::' + self.write_key)
        if res.status_code == 200:
            return res.json()

    def submit(self, name, values):
        """ Submit prediction scenarios
        :param name:      str         Examples:    cop.json   z1~cop.json   z2~cop~qp.json
        :param write_key: str         Example:    "5263ee89-e34e-44dc-8b91-445b302b043e"
        :param values:    [ float ]
        :return: bool
        """
        assert len(values)==self.num_predictions
        comma_sep_values = ",".join([ str(v) for v in values ] )
        res = requests.put(self.base_url + 'submit/' + name, data={'delay':self.delays[0], 'write_key': self.write_key, 'values': comma_sep_values})
        return res.status_code==200

    def cancel(self, name):
        """ Withdraw scenarios """
        codes = list()
        for delay in self.delays:
            res = requests.delete(self.base_url + 'submit/'+name, params={'delay':delay} )
            codes.append(res.status_code)
        return all([ c==200 for c in codes ] )



import uuid, re, sys, requests

CONFIG  = requests.get('https://www.microprediction.com/config.json').json()
MIN_LEN = CONFIG['min_len']

class NameConventions(object):

    @staticmethod
    def sep():
        return '::'

    @staticmethod
    def is_plain_name(name: str):
        return NameConventions.is_valid_name(name) and not "~" in name

    @staticmethod
    def is_valid_name(name: str):
        name_regex = re.compile(r'^[-a-zA-Z0-9_~.:]{1,200}\.[json,html]+$', re.IGNORECASE)
        return (re.match(name_regex, name) is not None) and (not NameConventions.sep() in name)

    @staticmethod
    def random_name():
        return str(uuid.uuid4()) + '.json'

class ValueConventions(object):

    @staticmethod
    def is_scalar_value(value):
        try:
            fv = float(value)
            return True
        except:
            return False

    @staticmethod
    def is_valid_value(value):
        return isinstance(value, (str, int, float)) and sys.getsizeof(value) < 100000

    @staticmethod
    def is_small_value(value):
        """ Somewhat arbitrary """
        return sys.getsizeof(value) < 1200


class StatsConventions():

    @staticmethod
    def percentile_abscissa():
        return [-8., -5., -4, -3, -2.5, -2.0, -1.5, -1.25, -1, -0.8, -0.6, -0.4, -0.2, -0.1, -0.05, -0.02, -0.01, 0.,
                0.01, 0.02, 0.05, 0.1, 0.2, 0.4, 0.6, 0.8, 1., 1.25, 1.5, 2.0, 2.5, 3., 4., 5., 8.]


class MicroConventions(NameConventions, ValueConventions, StatsConventions):
    pass





